from upgini.metadata import ModelTaskType, RuntimeParameters
from typing import Optional, Dict, Any
import logging
from upgini.resource_bundle import bundle


def get_runtime_params_custom_loss(
    loss: str,
    model_task_type: ModelTaskType,
    runtime_parameters: RuntimeParameters,
    logger: Optional[logging.Logger] = None,
) -> RuntimeParameters:
    if logger is None:
        logger = logging.getLogger()
    if loss is not None:
        selection_loss_reg = [
            "regression",
            "regression_l1",
            "huber",
            "poisson",
            "quantile",
            "mape",
            "mean_absolute_percentage_error",
            "gamma",
            "tweedie",
        ]
        selection_loss_binary = ["binary"]
        selection_loss_multi_clf = ["multiclass", "multiclassova", "multiclass_ova", "ova", "ovr"]
        use_custom_loss = (
            True
            if (
                (model_task_type == ModelTaskType.REGRESSION)
                and (loss in selection_loss_reg)
                or (model_task_type == ModelTaskType.BINARY)
                and (loss in selection_loss_binary)
                or (model_task_type == ModelTaskType.MULTICLASS)
                and (loss in selection_loss_multi_clf)
            )
            else False
        )

        if use_custom_loss:
            runtime_parameters.properties["lightgbm_params_preselection.objective"] = loss
            runtime_parameters.properties["lightgbm_params_base.objective"] = loss
            runtime_parameters.properties["lightgbm_params_segment.objective"] = loss
            msg = bundle.get("loss_selection_info").format(loss)
            logger.info(msg)
            print(msg)
        else:
            msg = bundle.get("loss_selection_warn").format(loss, model_task_type)
            logger.warning(msg)
            print(msg)

    return runtime_parameters


def get_additional_params_custom_loss(
    loss: str, model_task_type: ModelTaskType, logger: Optional[logging.Logger] = None
) -> Dict[str, Any]:
    if logger is None:
        logger = logging.getLogger()

    output_params = {}
    if loss is not None:
        calculation_loss_reg_map = {
            "regression": "RMSE",
            "poisson": "Poisson",
            "quantile": "Quantile",
            "mape": "MAPE",
            "mean_absolute_percentage_error": "MAPE",
        }
        calculation_loss_binary_map = {"binary": "Logloss"}
        calculation_loss_multi_clf_map = {
            "multiclass": "MultiClass",
            "multiclassova": "MultiClassOneVsAll",
            "multiclass_ova": "MultiClassOneVsAll",
            "ova": "MultiClassOneVsAll",
            "ovr": "MultiClass",
        }
        use_custom_loss = (
            True
            if (
                (model_task_type == ModelTaskType.REGRESSION)
                and (loss in calculation_loss_reg_map.keys())
                or (model_task_type == ModelTaskType.BINARY)
                and (loss in calculation_loss_binary_map.keys())
                or (model_task_type == ModelTaskType.MULTICLASS)
                and (loss in calculation_loss_multi_clf_map.keys())
            )
            else False
        )

        if use_custom_loss:
            if model_task_type == ModelTaskType.REGRESSION:
                output_params["loss_function"] = calculation_loss_reg_map[loss]
            elif model_task_type == ModelTaskType.BINARY:
                output_params["loss_function"] = calculation_loss_binary_map[loss]
            elif model_task_type == ModelTaskType.MULTICLASS:
                output_params["loss_function"] = calculation_loss_multi_clf_map[loss]

            msg = bundle.get("loss_calc_metrics_info").format(loss)
            logger.info(msg)
            print(msg)
        else:
            msg = bundle.get("loss_calc_metrics_warn").format(loss, model_task_type)
            logger.warning(msg)
            print(msg)

    return output_params



import re

import pandas as pd
import pytest
from sklearn.linear_model import LogisticRegression
from sklearn.model_selection import cross_val_score

from upgini.resource_bundle import bundle
from upgini.utils.blocked_time_series import BlockedTimeSeriesSplit


def _prepare_data():
    df = pd.read_csv("tests/test_data/binary/data2.csv.gz", compression="gzip")
    X, y = df[["feature1", "feature2"]], df["target"]
    cv = BlockedTimeSeriesSplit(n_splits=5, test_size=0.25)
    df_etalon_stat = pd.read_csv("tests/test_data/binary/blocked_ts_logic.csv")
    model = LogisticRegression(random_state=0)

    return X, y, cv, df_etalon_stat, model


def test_bts_split_logic():
    X, y, cv, df_etalon_stat, _ = _prepare_data()
    df_stat = []
    for _, (train_idx, test_idx) in enumerate(cv.split(X, y)):
        df_stat.append([len(train_idx), min(train_idx), max(train_idx), len(test_idx), min(test_idx), max(test_idx)])
    columns = ["train_len", "train_min", "train_max", "test_len", "test_min", "test_max"]
    df_stat = pd.DataFrame(df_stat, columns=columns)

    assert df_stat.equals(df_etalon_stat)


def test_bts_metrics():
    X, y, cv, _, model = _prepare_data()
    cv_result = set(cross_val_score(model, X, y, cv=cv, scoring="roc_auc"))
    assert cv_result == {
        0.4559664254320743,
        0.4767320313326982,
        0.4811855209016638,
        0.48947924927306374,
        0.5150543675843606,
    }


def test_bts_exceptions():
    X, y, cv, _, model = _prepare_data()

    X_short, y_short = X.iloc[:3, :], y.iloc[:3]
    with pytest.raises(
        expected_exception=ValueError, match=re.escape(bundle.get("timeseries_splits_more_than_samples").format(5, 3))
    ):
        _ = cross_val_score(model, X_short, y_short, cv=cv)

    X_short, y_short = X.iloc[:10, :], y.iloc[:10]
    with pytest.raises(expected_exception=ValueError, match=re.escape(bundle.get("timeseries_invalid_test_size"))):
        _ = cross_val_score(model, X_short, y_short, cv=cv)

    with pytest.raises(
        expected_exception=ValueError,
        match=re.escape(bundle.get("timeseries_invalid_split_type").format(5.5, "<class 'float'>")),
    ):
        _ = BlockedTimeSeriesSplit(n_splits=5.5, test_size=0.2)  # type: ignore

    with pytest.raises(
        expected_exception=ValueError,
        match=re.escape(bundle.get("timeseries_invalid_split_count").format(0)),
    ):
        _ = BlockedTimeSeriesSplit(n_splits=0, test_size=0.2)

    with pytest.raises(
        expected_exception=ValueError, match=re.escape(bundle.get("timeseries_invalid_test_size_type").format(2))
    ):
        _ = BlockedTimeSeriesSplit(n_splits=5, test_size=2)



import logging
from typing import Dict, List, Optional

import pandas as pd
from requests import get

from upgini.metadata import SearchKey
# from upgini.resource_bundle import bundle
# from upgini.utils.track_info import get_track_metrics


class IpToCountrySearchKeyConverter:

    url = "http://ip-api.com/json/{}"

    def __init__(
        self,
        search_keys: Dict[str, SearchKey],
        logger: Optional[logging.Logger] = None,
    ):
        self.search_keys = search_keys
        if logger is not None:
            self.logger = logger
        else:
            self.logger = logging.getLogger()
            self.logger.setLevel("FATAL")
        self.generated_features: List[str] = []

    def _get_country_code(self, ip: str) -> str:
        try:
            response = get(self.url.format(ip)).json()
            return response["countryCode"]
        except Exception:
            return None

    def convert(self, df: pd.DataFrame) -> pd.DataFrame:
        # TODO temporary turn off country detection

        # track_metrics = get_track_metrics()
        # if (
        #     track_metrics is not None
        #     and "ip" in track_metrics
        #     and track_metrics["ip"] is not None
        #     and track_metrics["ip"] != "0.0.0.0"
        # ):
        #     country_code = self._get_country_code(track_metrics["ip"])  # TODO check that country_code is not None!
        #     msg = bundle.get("country_auto_determined").format(country_code)
        #     print(msg)
        #     self.logger.info(msg)
        #     df["country_code"] = country_code
        #     self.search_keys["country_code"] = SearchKey.COUNTRY

        return df
